@@ -14,11 +14,7 @@ from aws_error_utils import catch_aws_error
 from gql.transport.aiohttp import AIOHTTPTransport
 from pycognito import Cognito
 
-from .. import (
-    _CREATE_AUDIT_RECORDS,
-    _GET_BULK_DATA_STORAGE_GQL,
-    _GET_NODE_GQL,
-)
+from .. import _CREATE_AUDIT_RECORDS, _GET_BULK_DATA_STORAGE_GQL, _GET_NODE_GQL
 from .. import BulkDataStorage as BaseBulkDataStorage
 from .. import Edge, Message, MessageType
 from .. import Node as BaseNode
@@ -68,16 +64,17 @@ class _AuditRecordQueue(asyncio.Queue):
 
             async for batch in batcher():
                 try:
-                    async with node._gql_client as session:
-                        await session.execute(
-                            _CREATE_AUDIT_RECORDS,
-                            variable_values=dict(
-                                name=node.name,
-                                tenant=node.tenant,
-                                messageType=message_type.name,
-                                auditRecords=batch,
-                            ),
-                        )
+                    async with node._lock:
+                        async with node._gql_client as session:
+                            await session.execute(
+                                _CREATE_AUDIT_RECORDS,
+                                variable_values=dict(
+                                    name=node.name,
+                                    tenant=node.tenant,
+                                    messageType=message_type.name,
+                                    auditRecords=batch,
+                                ),
+                            )
                 except asyncio.CancelledError:
                     cancelled.set()
                 except Exception:
@@ -129,13 +126,14 @@ class _BulkDataStorageQueue(asyncio.Queue):
                     bulk_data_storages: list[dict] = list()
                     try:
                         await self.__fill.wait()
-                        async with node._gql_client as session:
-                            bulk_data_storages = (
-                                await session.execute(
-                                    _GET_BULK_DATA_STORAGE_GQL,
-                                    variable_values={"tenant": node.tenant},
-                                )
-                            )["GetBulkDataStorage"]
+                        async with node._lock:
+                            async with node._gql_client as session:
+                                bulk_data_storages = (
+                                    await session.execute(
+                                        _GET_BULK_DATA_STORAGE_GQL,
+                                        variable_values={"tenant": node.tenant},
+                                    )
+                                )["GetBulkDataStorage"]
                     except asyncio.CancelledError:
                         cancelled.set()
                     except Exception:
@@ -155,18 +153,6 @@ class _BulkDataStorageQueue(asyncio.Queue):
         return bulk_data_storage if not bulk_data_storage.expired else await self.get()
 
 
-class _CognitoAIOHTTPTransport(AIOHTTPTransport):
-    def __init__(self, cognito: Cognito, url: str, **kwargs: Any) -> None:
-        self._cognito = cognito
-        super().__init__(url, **kwargs)
-
-    def __getattribute__(self, name: str) -> Any:
-        if name == "headers":
-            self._cognito.check_token()
-            return dict(Authorization=self._cognito.access_token)
-        return super().__getattribute__(name)
-
-
 class _DeleteMessageQueue(asyncio.Queue):
     def __init__(self, edge: Edge, node: Node) -> None:
         super().__init__()
@@ -391,6 +377,18 @@ class _TargetMessageQueue(asyncio.Queue):
         return await super().get()
 
 
+class CognitoAIOHTTPTransport(AIOHTTPTransport):
+    def __init__(self, cognito: Cognito, url: str, **kwargs: Any) -> None:
+        self._cognito = cognito
+        super().__init__(url, **kwargs)
+
+    def __getattribute__(self, name: str) -> Any:
+        if name == "headers":
+            self._cognito.check_token()
+            return dict(Authorization=self._cognito.access_token)
+        return super().__getattribute__(name)
+
+
 class Node(BaseNode):
     def __init__(
         self,
@@ -407,7 +405,7 @@ class Node(BaseNode):
         super().__init__(
             appsync_endpoint=appsync_endpoint,
             client_id=client_id,
-            gql_transport_cls=_CognitoAIOHTTPTransport,
+            gql_transport_cls=CognitoAIOHTTPTransport,
             name=name,
             password=password,
             tenant=tenant,
@@ -417,9 +415,14 @@ class Node(BaseNode):
         )
         self.__audit_records_queues: dict[str, _AuditRecordQueue] = dict()
         self.__bulk_data_storage_queue: _BulkDataStorageQueue = None
+        self.__lock: asyncio.Lock = None
         self.__source_message_receivers: list[_SourceMessageReceiver] = list()
         self.__target_message_queues: dict[str, _TargetMessageQueue] = dict()
 
+    @property
+    def _lock(self) -> asyncio.Lock:
+        return self.__lock
+
     def audit_message(
         self,
         /,
@@ -485,14 +488,16 @@ class Node(BaseNode):
 
     async def start(self) -> None:
         getLogger().info(f"Starting Node {self.name}")
+        self.__lock = asyncio.Lock()
         self.__bulk_data_storage_queue = _BulkDataStorageQueue(self)
-        async with self._gql_client as session:
-            data: dict[str, Union[str, dict]] = (
-                await session.execute(
-                    _GET_NODE_GQL,
-                    variable_values=dict(name=self.name, tenant=self.tenant),
-                )
-            )["GetNode"]
+        async with self.__lock:
+            async with self._gql_client as session:
+                data: dict[str, Union[str, dict]] = (
+                    await session.execute(
+                        _GET_NODE_GQL,
+                        variable_values=dict(name=self.name, tenant=self.tenant),
+                    )
+                )["GetNode"]
         self.config = (
             json.loads(data["tenant"].get("config") or "{}")
             | json.loads((data.get("app") or dict()).get("config") or "{}")
