@@ -5,7 +5,7 @@ from datetime import datetime, timezone
 from gzip import GzipFile
 from io import BytesIO
 from queue import Empty, Queue
-from threading import Event, Thread
+from threading import Event, RLock, Thread
 from time import sleep
 from typing import TYPE_CHECKING, Any, BinaryIO, Generator, Union
 from uuid import uuid4
@@ -14,11 +14,7 @@ import dynamic_function_loader
 from aws_error_utils import catch_aws_error
 from requests import post
 
-from .. import (
-    _CREATE_AUDIT_RECORDS,
-    _GET_BULK_DATA_STORAGE_GQL,
-    _GET_NODE_GQL,
-)
+from .. import _CREATE_AUDIT_RECORDS, _GET_BULK_DATA_STORAGE_GQL, _GET_NODE_GQL
 from .. import BulkDataStorage as BaseBulkDataStorage
 from .. import Edge, LambdaEvent, Message, MessageType
 from .. import Node as BaseNode
@@ -49,16 +45,17 @@ class _AuditRecordQueue(Queue):
                 if not batch:
                     continue
                 try:
-                    with node._gql_client as session:
-                        session.execute(
-                            _CREATE_AUDIT_RECORDS,
-                            variable_values=dict(
-                                name=node.name,
-                                tenant=node.tenant,
-                                messageType=message_type.name,
-                                auditRecords=batch,
-                            ),
-                        )
+                    with node._lock:
+                        with node._gql_client as session:
+                            session.execute(
+                                _CREATE_AUDIT_RECORDS,
+                                variable_values=dict(
+                                    name=node.name,
+                                    tenant=node.tenant,
+                                    messageType=message_type.name,
+                                    auditRecords=batch,
+                                ),
+                            )
                 except Exception:
                     getLogger().exception("Error creating audit records")
                 finally:
@@ -96,11 +93,12 @@ class _BulkDataStorageQueue(Queue):
             while True:
                 self.__fill.wait()
                 try:
-                    with node._gql_client as session:
-                        bulk_data_storages: list[dict] = session.execute(
-                            _GET_BULK_DATA_STORAGE_GQL,
-                            variable_values={"tenant": node.tenant},
-                        )["GetBulkDataStorage"]
+                    with node._lock:
+                        with node._gql_client as session:
+                            bulk_data_storages: list[dict] = session.execute(
+                                _GET_BULK_DATA_STORAGE_GQL,
+                                variable_values={"tenant": node.tenant},
+                            )["GetBulkDataStorage"]
                 except Exception:
                     getLogger().exception("Error getting bulk data storage")
                 else:
@@ -210,9 +208,13 @@ class Node(BaseNode):
         )
         self.__bulk_data_storage_queue = _BulkDataStorageQueue(self)
         self.__audit_records_queues: dict[str, _AuditRecordQueue] = dict()
-        self.__stop = Event()
+        self.__lock = RLock()
         self.__target_message_queues: dict[str, _TargetMessageQueue] = dict()
 
+    @property
+    def _lock(self) -> RLock:
+        return self.__lock
+
     def audit_message(
         self,
         /,
@@ -245,7 +247,6 @@ class Node(BaseNode):
         pass
 
     def join(self) -> None:
-        self.__stop.wait()
         for target_message_queue in self.__target_message_queues.values():
             target_message_queue.join()
         for audit_records_queue in self.__audit_records_queues.values():
@@ -269,12 +270,12 @@ class Node(BaseNode):
 
     def start(self) -> None:
         getLogger().info(f"Starting Node {self.name}")
-        self.__stop.clear()
-        with self._gql_client as session:
-            data: dict[str, Union[str, dict]] = session.execute(
-                _GET_NODE_GQL,
-                variable_values=dict(name=self.name, tenant=self.tenant),
-            )["GetNode"]
+        with self._lock:
+            with self._gql_client as session:
+                data: dict[str, Union[str, dict]] = session.execute(
+                    _GET_NODE_GQL,
+                    variable_values=dict(name=self.name, tenant=self.tenant),
+                )["GetNode"]
         self.config = (
             json.loads(data["tenant"].get("config") or "{}")
             | json.loads((data.get("app") or dict()).get("config") or "{}")
@@ -315,7 +316,7 @@ class Node(BaseNode):
         }
 
     def stop(self) -> None:
-        self.__stop.set()
+        pass
 
 
 class _DeleteMessageQueue(Queue):
@@ -460,14 +461,17 @@ class AppNode(Node):
             username=username,
         )
         self.__source_message_receivers: list[_SourceMessageReceiver] = list()
+        self.__stop = Event()
 
     def join(self) -> None:
+        self.__stop.wait()
         for app_node_receiver in self.__source_message_receivers:
             app_node_receiver.join()
         super().join()
 
     def start(self) -> None:
         super().start()
+        self.__stop.clear()
         self.__source_message_receivers = [
             _SourceMessageReceiver(edge, self) for edge in self._sources
         ]
@@ -477,9 +481,9 @@ class AppNode(Node):
         self.join()
 
     def stop(self) -> None:
+        self.__stop.set()
         for app_node_receiver in self.__source_message_receivers:
             app_node_receiver.stop()
-        super().stop()
 
 
 class LambdaNode(Node):
@@ -505,7 +509,7 @@ class LambdaNode(Node):
             user_pool_id=user_pool_id,
             username=username,
         )
-        super().start()
+        self.start()
         self.__queue_name_to_source = {
             edge.queue.split("/")[-1:][0]: edge.name for edge in self._sources
         }
